/*
Copyright © 2021 da/net Research Group <danet@h-da.de>
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

package cmd

import (
	"fmt"

	"code.fbi.h-da.de/danet/gosdn/controller/api"
	"github.com/pterm/pterm"
	"github.com/spf13/cobra"
	"github.com/spf13/viper"
)

// subGetCmd represents the get one subscription command.
var subGetCmd = &cobra.Command{
	Use:   "get",
	Short: "Fetches details about one specific currently running gNMI subscriptions.",
	Long:  "Fetches details about one specific currently running gNMI subscriptions.",
	Run: func(cmd *cobra.Command, args []string) {
		spinner, _ := pterm.DefaultSpinner.Start("Fetching subscription")

		resp, err := api.Get(createContextWithAuthorization(), viper.GetString("controllerAPIEndpoint"), subUUID)
		if err != nil {
			pterm.Error.Println(err)
			return
		}
		spinner.Success()

		data1 := pterm.TableData{[]string{"SubUUID", "MneID", "MneName", "PndID"}}
		data1 = append(data1, []string{resp.Subscriptions.Subid, resp.Subscriptions.Mneid, resp.Subscriptions.MneName, resp.Subscriptions.Pid})
		err = pterm.DefaultTable.WithHasHeader().WithData(data1).Render()
		if err != nil {
			return
		}

		pterm.Print(pterm.FgLightCyan.Sprintf("Subscribe Options\n"))
		data2 := pterm.TableData{[]string{"GnmiMode", "GnmiStreamMode", "SampleInterval (ns)"}}
		data2 = append(data2, []string{resp.Subscriptions.SubscribeOptions.GnmiMode, resp.Subscriptions.SubscribeOptions.GnmiStreamMode, fmt.Sprint(resp.Subscriptions.SubscribeOptions.SampleInterval)})
		err = pterm.DefaultTable.WithHasHeader().WithData(data2).Render()
		if err != nil {
			return
		}

		data3 := pterm.TableData{[]string{"Paths"}}
		for _, path := range resp.Subscriptions.Paths {
			data3 = append(data3, path.Elem)
		}

		err = pterm.DefaultTable.WithHasHeader().WithData(data3).Render()
		if err != nil {
			return
		}
	},
	PostRun: func(cmd *cobra.Command, args []string) {
		// Necessary for prompt mode. The flag variables have to be resetted,
		// since in prompt mode the program keeps running.
		subUUID = ""
	},
}

func init() {
	subCmd.AddCommand(subGetCmd)

	subGetCmd.Flags().StringVar(&subUUID, "uuid", "", "UUID of the subcription")
}
